#!/usr/bin/env python3

import requests
import tomlkit
import urllib3
urllib3.disable_warnings()

class StatusPage:
    def get_api_result(self):
        if self.services:
            return self.services

        headers = {
            'Accept': 'application/json',
            'X-HTTP-Method-Override': 'GET'
        }

        requestbody = {
            "attrs": [ "name", "state", "last_check_result", "host_name", "display_name" ],
            "joins": [ "host.name", "host.state", "host.last_check_result", "host.vars" ],
            "filter": self.config['filters']['services'],
        }

        r = requests.get(
            '{}/v1/objects/services'.format(self.config['icinga2_api']['baseurl']),
            headers=headers,
            json=requestbody,
            auth=(self.config['icinga2_api']['username'], self.config['icinga2_api']['password']),
            verify=False
        )

        if (r.status_code == 200):
            self.services = r.json()['results']
        else:
            r.raise_for_status()

        return self.services


    def prettify(self, text):
        for search, replace in self.config.get('prettify', {}).items():
            text = text.replace(search, replace)

        return text


    def render_services_per_host(self, host):
        services = []
        state_to_design_mapping = [
            ('success', 'OK'),
            ('warning', 'WARNING'),
            ('danger', 'CRITICAL'),
            ('info', 'UNKNOWN'),
        ]
        card_header = ''

        services_template = """
                <li class="list-group-item d-flex justify-content-between align-items-center">
                    {}
                    <span class="badge badge-{}">{}</span>
                </li>
            """
        services_hostname_template = """
                <div id="{0}" class="card-header d-flex justify-content-between align-items-center">
                    <h4><a href="#{0}">{1}</a></h4>
                    <span class="badge badge-{2}">{3}</span>
                </div>"""

        for service in sorted(self.get_api_result(), key=lambda x: x['attrs']['display_name']):
            if service['attrs']['host_name'] == host:
                state = int(service['attrs']['state'])
                if state in (1, 2):
                    self.ragecounter += state

                services.append(services_template.format(
                    self.prettify(service['attrs']['display_name']),
                    state_to_design_mapping[state][0],
                    state_to_design_mapping[state][1],
                ))

                if not card_header:
                    if service['joins']['host']['state'] == 0:
                        card_header = services_hostname_template.format(
                            host,
                            service['joins']['host']['vars']['pretty_name'],
                            'success',
                            'UP',
                        )
                    else:
                        card_header = services_hostname_template.format(
                            host,
                            service['joins']['host']['vars']['pretty_name'],
                            'danger',
                            'DOWN',
                        )
                        self.ragecounter += 100

        with open("services_template.html", "r") as f:
            htmlTemplate = f.read()

        htmlOutput = htmlTemplate.format(
            card_header = card_header,
            services = ''.join(services),
        )
        return htmlOutput


    def render_service_details(self):
        # generate list of hosts by scanning services for unique host_name
        host_names = {}
        for service in self.get_api_result():
            host_names[service['joins']['host']['vars']['pretty_name']] = service['attrs']['host_name']

        # render html for each host_name
        html_output = []
        # Can't use .values() here, since we want to sort by prettyname
        for prettyname, hostname in sorted(host_names.items()):
            html_output.append(self.render_services_per_host(hostname))

        return ''.join(html_output)


    def render_index_html(self, service_details):
        with open("template.html", "r") as f:
            htmlTemplate = f.read()
        if self.ragecounter == 0:
            mood = '🆗'
        elif self.ragecounter < 10:
            mood = '🚨'
        else:
            mood = '🔥'
        htmlOutput = htmlTemplate.format(
            services = service_details,
            mood = mood,
        )

        with open(self.config['output']['filename'], "w") as f:
            f.write(htmlOutput)


    def __init__(self):
        self.config = tomlkit.loads(open('config.toml').read())
        self.services = {}
        self.ragecounter = 0

if __name__ == "__main__":
    page = StatusPage()
    service_details = page.render_service_details()
    page.render_index_html(service_details)
